/**
 * UTILITY functions
 */
 	
/**
 * A utility function to a collection of resource and build divs for each resource in 
 * the container provided.
 * @param {Object} url
 * @param {Object} containerDivId
 * @param {Object} postProcessor
 * @param {Object} displayItems
 */
function getCollection(url, containerDivId, postProcessor, displayItems) {
	console.debug("Calling getCollection with " + url);
	var request = {};
	request.url = url;
	request.handleAs = "json";
	
	request.load = function(data, ioArgs) {
		var element, divNode, containerDiv;
		containerDiv = dojo.byId(containerDivId);
		if (!containerDiv) {
			containerDiv = dojo.widget.byId(containerDivId);
		}
		clearNode(containerDiv);
		for (element in data) {
			divNode = buildDiv(data[element], containerDiv, postProcessor, displayItems);
		}
		console.debug(ioArgs);
		console.debug("getCollection: success");
		return data;
	};
	
	request.error = function (data) {
			console.debug("getCollection: Got Error from iobind");
	};
	request.timeout = 20000;
	
	dojo.xhrGet(request);
}

/**
 * Remove any children from the provided dom node. The expectation is that the children of this node is dynamically created and needs to be 
 * cleared when state changes.
 * @param {Object} node
 */			
function clearNode(node) {
	var children = node.childNodes;
	var length = children.length;
	for (var ix = length -1; ix >= 0; ix--) {
		node.removeChild(children[ix]);
	}
}

/**
 * Remove any children with matching zero id (_zero_id attribute) from the provided dom node. 
 * @param {Object} node
 * @param (String) zero id
 */			
function removeChildByZeroId(node, zId) {
	var children = node.childNodes;
	var length = children.length;
	for (var ix = length -1; ix >= 0; ix--) {
		if (children[ix]._zero_id == zId) {
			node.removeChild(children[ix]);
			// Assume that there is just one child with that id in this node
			break;
		}
	}
}

/**
 * Build and attach a div element out of the json data into the container div
 * @param {Object} jsonDataForDiv Javascript object which has an id attribute which will become the id of the new div. 
 * 					Also any other attributes in the object will be prefixed with _zero in the div node.
 * @param {Object} containerDivNode The node that will host the newly created div
 * @param {Object} postProcessor A postprocessor function that will be invoked with the div that is created as an argument. (optional)
 * @param {Array} displayItems An array of attribute names that are concatenated as a text element into the div. (optional)
 */
function buildDiv(jsonDataForDiv, containerDivNode, postProcessor, displayItems) {
	var divNode;
	var attr, displayText= "", classAttribute;
	divNode = document.createElement("div");
	if (jsonDataForDiv.id != undefined) {
		divNode.id = jsonDataForDiv.id;
	}
	// Allow for omitted postProcessor
	if (!dojo.isFunction(postProcessor)) {
		displayItems = postProcessor;
		postProcessor = null;
	}
	if (displayItems) {
		if (displayItems != "_AS_IS_") {
			for (attr in displayItems) {
				displayText += jsonDataForDiv[displayItems[attr]] + " ";
			}
		} else {
			displayText = jsonDataForDiv;
		}
		textNode = document.createTextNode(displayText);
		divNode.appendChild(textNode);
	}
	
	
	for (attr in jsonDataForDiv) {
		divNode["_zero_" + attr] = jsonDataForDiv[attr];
	}
	if (postProcessor) {
		(postProcessor)(divNode);
	}
	containerDivNode.appendChild(divNode);
	return divNode;
}					


/** 
 * Form utility function to copy values from an object into another (if it is a form, you can also set readonly attributes
 * and the submit (submitbut) and cancel (cancelbut) button values
 */
function setFormValues(targetObj, srcObj, valueNames, readOnly, submitFn, cancelFn) {
/*
	for(name in valueNames) {
		var valueName = valueNames[name];
		targetObj[valueName].value = srcObj[valueName];
		targetObj[valueName].readOnly = readOnly;
	}
	var containerObj = targetObj;
	
*/
	targetObj.setValues(srcObj);
	var containerObj = targetObj.containerNode;
	if (submitFn) {
		containerObj.submitbut.onclick = submitFn;
		containerObj.submitbut.style.display = "block";
	} else if (containerObj.submitbut != undefined && readonly){ // readonly form should not have a submit button.
		containerObj.submitbut.style.display = "none";
	}
	if (cancelFn) {
		containerObj.cancelbut.onclick = cancelFn;
		containerObj.cancelbut.style.display = "block";
	} else if (containerObj.cancelbut != undefined && readOnly) { // readonly form should not have a submit button.
		containerObj.cancelbut.style.display = "none";
	}
} 
			
/**
 * Form Utility function to get form values
 */
function getFormValues(srcObj, valueNames) {
/*
	var targetObj = {};
	for (var name in valueNames) {
		var valueName = valueNames[name];
		targetObj[valueName] = srcObj[valueName].value;
	} 
	return targetObj;
*/
	console.debug(srcObj);
	return srcObj.getValues();
}

/**
 * standin for dojo.lfx.highlight
 */
 function highlight(node) {
    var anim = dojo.animateProperty({
      node: node,
      duration: 700,
      properties: {
            backgroundColor: { start:'rgb(255, 255, 153)', end: "#ff7" }
      }
    });
    anim.play();
}
 


/** 
 * Utility to prepend context to URL
 */
 function addContextRoot(url) {
 	return _zeroContext + url;
 }
